
#include "stdio.h"
#include <string.h>

#include "hostRecvSlaveDataParse.h"
#include "hostQueueUart.h"

#include "mcu_common.h"

static void slaveStateMachine(void);

/* ״̬ */
static uint8_t slaveAnalysisWait(void);
static uint8_t slaveAnalysisStartFlagSL(void);
static uint8_t slaveAnalysisPortSL(void);
static uint8_t slaveAnalysisLengthSL(void);
static uint8_t slaveAnalysisEndFlagSL(void);


/* ״̬״̬ */
uartStateMachine slaveState = wait;

/**
  * @brief  ״̬
  * @param 
  * @retval 
  */
void slaveStateMachine(void)
{
    if (slaveState == wait) {
        if (slaveAnalysisWait() == TRUE) {
            mcuUartRxTime = xTaskGetTickCount();
        }
    }

    else if (slaveState == startFlagSL) {
        slaveAnalysisStartFlagSL();
    }

    else if (slaveState == portSL) {
        slaveAnalysisPortSL();
    }

    else if (slaveState == lengthSL) {
        slaveAnalysisLengthSL();
    }

    else if (slaveState == endFlagSL) {
        slaveAnalysisEndFlagSL();
    }
}

/**
  * @brief  ״̬ wait
  * @param
  * @retval 0:ʧ
            1:ɹ
  */
#define waitMaxLen 2
uint8_t slaveAnalysisWait(void)
{
    if (mcuUartRxBufferIndex >= 2) {
        /* ͸֡ͷ */
        if (mcuUartRxBuffer[0] == 'S' && mcuUartRxBuffer[1] == 'L') {
            // log_info("startFlagSL\n");
            state = startFlagSL;
            return TRUE;
        }
    }

    if (mcuUartRxBufferIndex < waitMaxLen) {
        return FALSE;
    }

    state = wait;
    mcuUartRxBufferIndex--;
    memcpy(mcuUartRxBuffer, mcuUartRxBuffer + 1, mcuUartRxBufferIndex);

    return FALSE;
}

/**
  * @brief  ״̬ startFlagSL
  * @param
  * @retval 0:ʧ
            1:ɹ
  */
uint8_t slaveAnalysisStartFlagSL(void)
{
    /* У˿ں */
    if (mcuUartRxBuffer[2] < 13) {
        // log_info("portSL\n");
        state = portSL;
        return TRUE;
    }

    // log_error("portSL %d \n", gw485RxBuffer[2]);
    state = wait;
    mcuUartRxBufferIndex--;
    memcpy(mcuUartRxBuffer, mcuUartRxBuffer + 1, mcuUartRxBufferIndex);
    return FALSE;
}

/**
  * @brief  ״̬ portSL
  * @param
  * @retval 0:ʧ
            1:ɹ
  */
#define PortSLMaxLen 5
uint8_t slaveAnalysisPortSL(void)
{
    /* ݰĶ */
    if (mcuUartRxBufferIndex >= PortSLMaxLen) {
        uint32_t tempLen = 0;
        tempLen = (mcuUartRxBuffer[3] << 8) | mcuUartRxBuffer[4];

        if (tempLen <= 1024) {
            // log_info("lengthSL\n");
            state = lengthSL;
            frameLength = 6 + tempLen;
            return TRUE;
        }
    }

    if (mcuUartRxBufferIndex < PortSLMaxLen) {
        return FALSE;
    }
    
    // log_error("lengthSL %d \n", (gw485RxBuffer[3] << 8) | gw485RxBuffer[4]);

    state = wait;
    mcuUartRxBufferIndex--;
    memcpy(mcuUartRxBuffer, mcuUartRxBuffer + 1, mcuUartRxBufferIndex);
    return FALSE;
}

/**
  * @brief  ״̬ lengthSL
  * @param
  * @retval 0:ʧ
            1:ɹ
  */
#define LengthSLMaxLen frameLength
uint8_t slaveAnalysisLengthSL(void)
{
    /* ֡β */
    if (mcuUartRxBufferIndex >= LengthSLMaxLen) {
        if (mcuUartRxBuffer[LengthSLMaxLen - 1] == 0x17) {
            // log_info("endFlagSL\n");
            state = endFlagSL;
            return TRUE;
        }
    }

    if (mcuUartRxBufferIndex < LengthSLMaxLen) {
        return FALSE;
    }
    // log_info("endFlagSL %d\n", gw485RxBuffer[LengthSLMaxLen - 1]);

    state = wait;
    mcuUartRxBufferIndex--;
    memcpy(mcuUartRxBuffer, mcuUartRxBuffer + 1, mcuUartRxBufferIndex);
    return FALSE;
}

/**
  * @brief  ״̬ endFlagSL
  * @param
  * @retval 0:ʧ
            1:ɹ
  */
uint8_t slaveAnalysisEndFlagSL(void)
{
    uint16_t tempLen = ((mcuUartRxBuffer[3] << 8) | mcuUartRxBuffer[4]) + 6;

    //ϵͳڴ治㣬ȥǰ
    if (xPortGetFreeHeapSize() < tempLen + 1024) {
        //buff
        state = wait;
        su806RxBufferIndex = 0;
        return 1;
    }

    uint8_t *Buff = (uint8_t *)pvPortMalloc(tempLen + hostQueueUartSendInfoSize);
    hostQueueUartSendInfo *sendBuff = (hostQueueUartSendInfo *)Buff;
    sendBuff->length = tempLen;
    sendBuff->data = Buff + hostQueueUartSendInfoSize;
    memcpy((char *)sendBuff->data, (char *)su806RxBuffer, tempLen);


    if (uxQueueSpacesAvailable(su806_uart_Queue)) {
        xQueueSend(su806_uart_Queue, &Buff, 10);
    }
    /* ޿ռ䣬ͷڴ棬˳ */
    else {
        vPortFree(Buff);
    }

    return 0;
}

void hostRecvSlaveDataParse(device_handle device)
{
    /* ÿκִ10ms */
    static uint32_t tickstart = 0U;
    tickstart = xTaskGetTickCount();

    /* 2Sδһ֡ݣ */
    if (getTickDiff(mcuUartRxTime) >= tick_2S) {
        mcuUartRxTime = xTaskGetTickCount();
        mcuUartRxBufferIndex = 0;
        state = wait;
    }

    while (uart_dev_char_present(device) == 1 && ((xTaskGetTickCount() - tickstart) < 5)) {
        mcuUartRxBuffer[mcuUartRxBufferIndex++] = uart_dev_in_char(device);
        slaveStateMachine();
    }

    if (uart_dev_char_present(device) != 1 && state != wait) {        
        slaveStateMachine();
    }
}

