
#include "slaveMcuDataParse.h"
#include "slaveMcuComm.h"

static void stateMachine(void);

/* ״̬ */
static uint8_t analysisWait(void);
static uint8_t analysisStartFlagSL(void);
static uint8_t analysisPortSL(void);
static uint8_t analysisLengthSL(void);
static uint8_t analysisEndFlagSL(void);

/**
  * @brief  ״̬
  * @param 
  * @retval 
  */
void stateMachine(void)
{
    if (state == wait) {
        if (analysisWait() == TRUE) {
            mcuUartRxTime = xTaskGetTickCount();
        }
    }

    else if (state == startFlagSL) {
        analysisStartFlagSL();
    }

    else if (state == portSL) {
        analysisPortSL();
    }

    else if (state == lengthSL) {
        analysisLengthSL();
    }

    else if (state == endFlagSL) {
        analysisEndFlagSL();
    }
}


/**
  * @brief  ״̬ wait
  * @param
  * @retval 0:ʧ
            1:ɹ
  */
#define waitMaxLen 2
uint8_t analysisWait(void)
{
    if (mcuUartRxBufferIndex >= 2) {
        /* ͸֡ͷ */
        if (mcuUartRxBuffer[0] == 'S' && mcuUartRxBuffer[1] == 'L') {
            // log_info("startFlagSL\n");
            state = startFlagSL;
            return TRUE;
        }
    }

    if (mcuUartRxBufferIndex < waitMaxLen) {
        return FALSE;
    }

    state = wait;
    mcuUartRxBufferIndex--;
    memcpy(mcuUartRxBuffer, mcuUartRxBuffer + 1, mcuUartRxBufferIndex);

    return FALSE;
}

/**
  * @brief  ״̬ startFlagSL
  * @param
  * @retval 0:ʧ
            1:ɹ
  */
uint8_t analysisStartFlagSL(void)
{
    /* У˿ں */
    if (mcuUartRxBuffer[2] < 13) {
        // log_info("portSL\n");
        state = portSL;
        return TRUE;
    }

    // log_error("portSL %d \n", gw485RxBuffer[2]);
    state = wait;
    mcuUartRxBufferIndex--;
    memcpy(mcuUartRxBuffer, mcuUartRxBuffer + 1, mcuUartRxBufferIndex);
    return FALSE;
}

/**
  * @brief  ״̬ portSL
  * @param
  * @retval 0:ʧ
            1:ɹ
  */
#define PortSLMaxLen 5
uint8_t analysisPortSL(void)
{
    /* ݰĶ */
    if (mcuUartRxBufferIndex >= PortSLMaxLen) {
        uint32_t tempLen = 0;
        tempLen = (mcuUartRxBuffer[3] << 8) | mcuUartRxBuffer[4];

        if (tempLen <= 1024) {
            // log_info("lengthSL\n");
            state = lengthSL;
            frameLength = 6 + tempLen;
            return TRUE;
        }
    }

    if (mcuUartRxBufferIndex < PortSLMaxLen) {
        return FALSE;
    }
    
    // log_error("lengthSL %d \n", (gw485RxBuffer[3] << 8) | gw485RxBuffer[4]);

    state = wait;
    mcuUartRxBufferIndex--;
    memcpy(mcuUartRxBuffer, mcuUartRxBuffer + 1, mcuUartRxBufferIndex);
    return FALSE;
}

/**
  * @brief  ״̬ lengthSL
  * @param
  * @retval 0:ʧ
            1:ɹ
  */
#define LengthSLMaxLen frameLength
uint8_t analysisLengthSL(void)
{
    /* ֡β */
    if (mcuUartRxBufferIndex >= LengthSLMaxLen) {
        if (mcuUartRxBuffer[LengthSLMaxLen - 1] == 0x17) {
            // log_info("endFlagSL\n");
            state = endFlagSL;
            return TRUE;
        }
    }

    if (mcuUartRxBufferIndex < LengthSLMaxLen) {
        return FALSE;
    }
    // log_info("endFlagSL %d\n", gw485RxBuffer[LengthSLMaxLen - 1]);

    state = wait;
    mcuUartRxBufferIndex--;
    memcpy(mcuUartRxBuffer, mcuUartRxBuffer + 1, mcuUartRxBufferIndex);
    return FALSE;
}

/**
  * @brief  ״̬ endFlagSL
  * @param
  * @retval 0:ʧ
            1:ɹ
  */
uint8_t analysisEndFlagSL(void)
{
    uint32_t tempLen = 0;
    tempLen = (mcuUartRxBuffer[3] << 8) | mcuUartRxBuffer[4];

    if (mcuUartRxBuffer[2] == SlaveMcu) {
        slaveFrameDataAnalyze(mcuUartRxBuffer, mcuUartRxBufferIndex);
        mcuUartRxBufferIndex = 0;
        state = wait;
        return 1;
    }

    //ϵͳڴ治㣬ȥǰ
    if (xPortGetFreeHeapSize() < tempLen + 1024) {
        goto err;
    }

    /* ڲͬͨ˿ڣĽṹ岻ͬ */
    uint8_t *Buff;

    Buff = (uint8_t *)pvPortMalloc(tempLen + slaveQueueUartSendInfoSize);
    slaveQueueUartSendInfo *sendBuff = (slaveQueueUartSendInfo *)Buff;
    sendBuff->length = tempLen;
    sendBuff->data = Buff + slaveQueueUartSendInfoSize;
    memcpy((char *)sendBuff->data, (char *)&mcuUartRxBuffer[5], tempLen);


    /* ͨͬĶ˿ڽݷ͵ͬĵط */
    if (mcuUartRxBuffer[2] == J1) {
        if (uxQueueSpacesAvailable(J1_485_Queue)) {
            xQueueSend(J1_485_Queue, &Buff, 10);
        }
        /* ޿ռ䣬ͷڴ棬˳ */
        else {
            vPortFree(Buff);
        }
    }
    else if (mcuUartRxBuffer[2] == J3) {
        if (uxQueueSpacesAvailable(J3_485_Queue)) {
            xQueueSend(J3_485_Queue, &Buff, 10);
        }
        /* ޿ռ䣬ͷڴ棬˳ */
        else {
            vPortFree(Buff);
        }
    }    
    else if (mcuUartRxBuffer[2] == J5) {
        if (uxQueueSpacesAvailable(J5_485_Queue)) {
            xQueueSend(J5_485_Queue, &Buff, 10);
        }
        /* ޿ռ䣬ͷڴ棬˳ */
        else {
            vPortFree(Buff);
        }
    }    
    else if (mcuUartRxBuffer[2] == J7) {
        if (uxQueueSpacesAvailable(J7_485_Queue)) {
            xQueueSend(J7_485_Queue, &Buff, 10);
        }
        /* ޿ռ䣬ͷڴ棬˳ */
        else {
            vPortFree(Buff);
        }
    }
    else if (mcuUartRxBuffer[2] == J8) {
        if (uxQueueSpacesAvailable(J8_485_Queue)) {
            xQueueSend(J8_485_Queue, &Buff, 10);
        }
        /* ޿ռ䣬ͷڴ棬˳ */
        else {
            vPortFree(Buff);
        }
    }
    else if (mcuUartRxBuffer[2] == J9) {
        if (uxQueueSpacesAvailable(J9_485_Queue)) {
            xQueueSend(J9_485_Queue, &Buff, 10);
        }
        /* ޿ռ䣬ͷڴ棬˳ */
        else {
            vPortFree(Buff);
        }
    }

err:
    //buff
    state = wait;
    mcuUartRxBufferIndex = 0;
    return 1;
}

/**
  * @brief  ģ鷢
  * @param  device ڽģ鷢ݵĴ豸
  * @retval
  */
void slaveMcuUartDataAnalysis(device_handle device)
{
    /* ÿκִ10ms */
    static uint32_t tickstart = 0U;
    tickstart = xTaskGetTickCount();

    /* 2Sδһ֡ݣ */
    if (getTickDiff(mcuUartRxTime) >= tick_2S) {
        mcuUartRxTime = xTaskGetTickCount();
        mcuUartRxBufferIndex = 0;
        state = wait;
    }

    while (uart_dev_char_present(device) == 1 && ((xTaskGetTickCount() - tickstart) < 5)) {
        mcuUartRxBuffer[mcuUartRxBufferIndex++] = uart_dev_in_char(device);
        stateMachine();
    }

    if (uart_dev_char_present(device) != 1 && state != wait) {        
        stateMachine();
    }
}





